from pydantic import BaseModel, Field
from datetime import datetime

class ValidityPeriod(BaseModel):
    from_date: datetime = Field(alias='fromDate')
    to_date: datetime = Field(alias='toDate')
    is_now: bool = Field(alias='isNow')

    model_config = {'populate_by_name': True}



from pydantic import BaseModel, Field
from datetime import datetime
from typing import Optional

class AdditionalProperties(BaseModel):
    category: Optional[str] = Field(None, alias='category')
    key:  Optional[str] = Field(None, alias='key')
    source_system_key:  Optional[str] = Field(None, alias='sourceSystemKey')
    value:  Optional[str] = Field(None, alias='value')
    modified: Optional[datetime] = Field(None, alias='modified')

    model_config = {'populate_by_name': True}



from pydantic import BaseModel, Field
from datetime import datetime

class PredictionTiming(BaseModel):
    countdown_server_adjustment: str = Field(alias='countdownServerAdjustment')
    source: datetime = Field(alias='source')
    insert: datetime = Field(alias='insert')
    read: datetime = Field(alias='read')
    sent: datetime = Field(alias='sent')
    received: datetime = Field(alias='received')

    model_config = {'populate_by_name': True}



from pydantic import BaseModel, Field
from datetime import datetime
from typing import Optional

class RouteSection(BaseModel):
    route_code: Optional[str] = Field(None, alias='routeCode')
    name: str = Field(alias='name')
    direction: Optional[str] = Field(None, alias='direction')
    origination_name: str = Field(alias='originationName')
    destination_name: str = Field(alias='destinationName')
    originator: str = Field(alias='originator')
    destination: str = Field(alias='destination')
    service_type: Optional[str] = Field(None, alias='serviceType')
    valid_to: Optional[datetime] = Field(None, alias='validTo')
    valid_from: Optional[datetime] = Field(None, alias='validFrom')

    model_config = {'populate_by_name': True}



from pydantic import BaseModel, Field
from typing import Optional

class OrderedRoute(BaseModel):
    name: str = Field(alias='name')
    naptan_ids: list[str] = Field(alias='naptanIds')
    service_type: str = Field(alias='serviceType')

    model_config = {'populate_by_name': True}


from pydantic import BaseModel, Field
from typing import Optional

from .crowding import Crowding

class Identifier(BaseModel):
    id: str = Field(alias='id')
    name: str = Field(alias='name')
    uri: str = Field(alias='uri')
    full_name: Optional[str] = Field(None, alias='fullName')
    identifier_type: str = Field(alias='type')
    crowding: Optional[Crowding] = Field(None, alias='crowding')
    status: str = Field(alias='status')

    model_config = {'populate_by_name': True}


from pydantic import BaseModel, Field, field_validator
from datetime import datetime

class ApiError(BaseModel):
    timestamp_utc: datetime = Field(alias='timestampUtc')
    exception_type: str = Field(alias='exceptionType')
    http_status_code: int = Field(alias='httpStatusCode')
    http_status: str = Field(alias='httpStatus')
    relative_uri: str = Field(alias='relativeUri')
    message: str = Field(alias='message')

    @field_validator('timestamp_utc', mode='before')
    def parse_timestamp(cls, v):
        return datetime.strptime(v, '%a, %d %b %Y %H:%M:%S %Z')

    model_config = {'populate_by_name': True}



from pydantic import BaseModel, Field
from typing import Optional

class PassengerFlow(BaseModel):
    time_slice: Optional[str] = Field(None, alias='timeSlice')
    value: Optional[int] = Field(None, alias='value')

    model_config = {'populate_by_name': True}


# portions of this code are from https://github.com/dhilmathy/TfL-python-api
# this code from 
# MIT License

# Copyright (c) 2018 Mathivanan Palanisamy
# Copyright (c) 2024 Rob Aleck

# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:

# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.

# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.


from .additional_properties import AdditionalProperties  # no dependencies
from .line_mode_group import LineModeGroup  # no dependencies
from .line_group import LineGroup  # no dependencies
from .stop_point import (
    StopPoint,
)  # dependent on Line (commented out), LineGroup, LineModeGroup, AdditionalProperties
from .stop_points_response import StopPointsResponse  # dependent on StopPoint

from .route_section_naptan_entry_sequence import (
    RouteSectionNaptanEntrySequence,
)  # dependent on StopPoint
from .affected_route import (
    AffectedRoute,
)  # dependent on RouteSectionNaptanEntrySequence
from .disruption import Disruption  # dependent on AffectedRoute and StopPoint

from .service_type import ServiceType  # no dependencies
from .route_section import RouteSection  # no dependencies

from .passenger_flow import PassengerFlow  # no dependencies
from .train_loading import TrainLoading  # no dependencies
from .crowding import Crowding  # dependent on PassengerFlow and TrainLoading

from .validity_period import ValidityPeriod  # no dependencies
from .line_status import LineStatus  # dependent on ValidityPeriod

from .line import (
    Line,
)  # dependent on Disruption, LineStatus, RouteSection, ServiceType, Crowding

from .prediction_timing import PredictionTiming  # no dependencies
from .prediction import Prediction  # dependent on PredictionTiming

from .api_error import ApiError  # no dependencies
from .mode import Mode  # no dependencies

from .matched_stop import MatchedStop  # no dependencies
from .ordered_route import OrderedRoute  # no dependencies
from .route_sequence import RouteSequence  # dependent on MatchedStop and OrderedRoute

__all__ = [
    "AdditionalProperties",
    "AffectedRoute",
    "ApiError",
    "Crowding",
    "Disruption",
    "Line",
    "LineGroup",
    "LineModeGroup",
    "LineStatus",
    "Mode",
    "MatchedStop",
    "OrderedRoute",
    "PassengerFlow",
    "Prediction",
    "PredictionTiming",
    "RouteSection",
    "RouteSectionNaptanEntrySequence",
    "RouteSequence",
    "ServiceType",
    "StopPoint",
    "StopPointsResponse",
    "TrainLoading",
    "ValidityPeriod",
]



from pydantic import BaseModel, Field
from typing import Optional
from datetime import datetime

from .matched_stop import MatchedStop
from .ordered_route import OrderedRoute

class RouteSequence(BaseModel):
    line_id: str = Field(alias='lineId')
    line_name: str = Field(alias='lineName')
    direction: str = Field(alias='direction')
    is_outbound_only: bool = Field(alias='isOutboundOnly')
    mode: str = Field(alias='mode')
    line_strings: list[str] = Field(alias='lineStrings')
    stations: list[MatchedStop] = Field(alias='stations')
    service_type: Optional[str] = Field(None, alias='serviceType')
    ordered_line_routes: list[OrderedRoute] = Field(alias='orderedLineRoutes')
    content_expires: Optional[datetime] = Field(None)
    
    model_config = {'populate_by_name': True}



from pydantic import BaseModel, Field
from typing import List, Optional
from datetime import datetime

from .stop_point import StopPoint

class StopPointsResponse(BaseModel):
    stop_points: Optional[List[StopPoint]] = Field(None, alias='stopPoints')
    page_size: int = Field(alias='pageSize')
    total: int = Field(alias='total')
    page: int = Field(alias='page')
    content_expires: Optional[datetime] = Field(None)

    model_config = {'populate_by_name': True}
